@@ -28,6 +28,7 @@ def gen_query_rpc_endpoint(
         tags: list[str] | None = None,
         deprecated: bool = False,
         operation_id: str = '',
+        allow_extra_params: bool = False,
 ) -> RpcEndpoint:
     """
     Generate RPC endpoint based on function signature:
@@ -58,6 +59,7 @@ def gen_json_rpc_endpoint(
         tags: list[str] | None = None,
         deprecated: bool = False,
         operation_id: str = '',
+        allow_extra_params: bool = False,
 ) -> RpcEndpoint:
     """
     Generate RPC endpoint based on function signature:
@@ -70,6 +72,7 @@ def gen_json_rpc_endpoint(
         unhandled_exception__answer_type=unhandled_exception__answer_type,
         arg_name_to_param=arg_name_to_param, answer_type=answer_type, answer_factory=answer_factory,
         summary=summary, description=description, tags=tags, deprecated=deprecated, operation_id=operation_id,
+        allow_extra_params=allow_extra_params
     )
 
 
@@ -92,6 +95,7 @@ def _gen_rpc_endpoint(
         tags: list[str] | None = None,
         deprecated: bool = False,
         operation_id: str = '',
+        allow_extra_params: bool = False,
 ) -> RpcEndpoint:
     path_param_names = _get_path_param_names(path)
     security_param_names = get_securities_arg_names(securities)
@@ -136,6 +140,13 @@ def _gen_rpc_endpoint(
             param = param_type(**parameter_kwargs)
         result_arg_name_to_param[arg.name] = param
 
+    if extra_params := arg_name_to_param.keys() - result_arg_name_to_param.keys():
+        if allow_extra_params:
+            for extra_param in extra_params:
+                result_arg_name_to_param[extra_param] = arg_name_to_param[extra_param]
+        else:
+            raise KeyError(f"arg_name_to_param has {extra_params=}, use allow_extra_params=True to add those")
+
     if signature.return_annotation == Signature.empty:
         raise TypeError("undefined return type")
 
