@@ -11,6 +11,7 @@ import dataclasses
 import inspect
 import json
 import logging
+import pathlib
 import traceback
 import typing
 import warnings
@@ -19,6 +20,7 @@ from functools import cached_property
 from typing import Callable, Any, Awaitable, TypeVar, Literal
 
 import http_tools
+import yarl
 from async_tools import acall
 from dict_caster.extras import first
 from http_tools import Answer, HttpServer, HttpStatusCode
@@ -58,6 +60,10 @@ class RpcEndpoint(typing.Generic[ResultType, AnswerType]):
     operation_id: str
     # преобразователь ответа обработчика в HTTP ответ
     answer_factory: Callable[[ResultType], AnswerType]
+    summary: str
+    description: str
+    tags: list[str]
+    deprecated: bool
 
     def __init__(
             self,
@@ -69,10 +75,22 @@ class RpcEndpoint(typing.Generic[ResultType, AnswerType]):
             arg_name_to_param: dict[str, CallParameter | type[RawBodyParameter]],
             answer_type: type[AnswerType],
             exception_type_to_answer_type: dict[type[Exception], type[ExceptionAnswer]] | None = None,
-            unhandled_exception__answer_type: type[Answer] = ExceptionAnswer[HttpStatusCode.InternalServerError],
+            unhandled_exception__answer_type: type[Answer] | None = ExceptionAnswer[HttpStatusCode.InternalServerError],
             operation_id: str = '',
             answer_factory: Callable[[ResultType], AnswerType] | None = None,
+            summary: str | None = None,
+            description: str | None = None,
+            tags: list[str] | None = None,
+            deprecated: bool = False,
     ) -> None:
+
+        # if typing.get_origin(func) == typing.Annotated:
+        #     args = typing.get_args(func)
+        #     func = args[0]
+        #     if len(args) > 1 and not summary:
+        #         summary = args[1]
+        #         assert isinstance(summary, str), 'Endpoint annotation MUST be str'
+
         object.__setattr__(self, 'func', func)
         object.__setattr__(self, 'method', method)
         object.__setattr__(self, 'path', path)
@@ -82,8 +100,12 @@ class RpcEndpoint(typing.Generic[ResultType, AnswerType]):
         object.__setattr__(self, 'answer_type', answer_type)
         object.__setattr__(self, 'exception_type_to_answer_type', exception_type_to_answer_type or {})
         object.__setattr__(self, 'unhandled_exception__answer_type', unhandled_exception__answer_type)
-        object.__setattr__(self, 'operation_id', operation_id or self.func.__name__)
+        object.__setattr__(self, 'operation_id', operation_id or func.__name__)
         object.__setattr__(self, 'answer_factory', answer_factory or answer_type)
+        object.__setattr__(self, 'summary', summary or '')
+        object.__setattr__(self, 'description', description or func.__doc__ or '')
+        object.__setattr__(self, 'tags', tags or [])
+        object.__setattr__(self, 'deprecated', deprecated)
         get_securities_arg_names(self.securities)
 
     @cached_property
@@ -119,7 +141,7 @@ class RpcEndpoint(typing.Generic[ResultType, AnswerType]):
 
         class_: type = type(self.operation_id, tuple(), {
             p.name: field(default=MISSING if p.default is not NoValue else p.default) for p in body_params})
-        class_.__annotations__ = {param.name: param.schema for param in body_params}
+        class_.__annotations__ = {param.name: param.annotated_schema for param in body_params}
         # noinspection PyTypeChecker
         return dataclasses.dataclass(kw_only=True)(class_)
 
@@ -145,7 +167,11 @@ class RpcEndpoint(typing.Generic[ResultType, AnswerType]):
             securities=self.securities,
             parameters=[param for param in self.non_body_parameters if isinstance(param, SpecParameter)],
             request_body=self.request_body,
-            code_to_answer=self.code_to_answer
+            code_to_answer=self.code_to_answer,
+            summary=self.summary,
+            description=self.description,
+            tags=self.tags,
+            deprecated=self.deprecated,
         )
 
     def gen_handler(self) -> Callable[[IncomingRequest], Awaitable[Answer | StreamAnswer]]:
@@ -227,7 +253,24 @@ class OpenApiServer:
     @dataclass
     class Config(OpenApiWrapper.Config):
         write_spec_to_file: str = ""  # empty means skip
-        spec_http_path: str = '/open_api_specification/get'  # empty means disable
+        http_prefix: str = '/open_api_specification/'
+        spec_http_path: str = 'get'  # empty means disable
+        ui_http_path: str = 'ui'  # empty means disable
+
+        def __post_init__(self):
+            if prefix := self.http_prefix:
+                raise_if(not prefix[0] == '/', ValueError(f"Http prefix MUST start with '/', got: {prefix}"))
+                raise_if(not prefix[-1] == '/', ValueError(f"Http prefix MUST end with '/', got: {prefix}"))
+
+    @property
+    def spec_path(self) -> str | None:
+        if self.config.spec_http_path:
+            return yarl.URL(self.config.http_prefix).join(yarl.URL(self.config.spec_http_path)).path
+
+    @property
+    def ui_path(self) -> str | None:
+        if self.config.ui_http_path:
+            return yarl.URL(self.config.http_prefix).join(yarl.URL(self.config.ui_http_path)).path
 
     def __init__(
             self, config: Config, context: Context,
@@ -236,8 +279,36 @@ class OpenApiServer:
         self.context = context
         self.config = config
         self.wrapper = wrapper_factory(config, context)
-        if path := self.config.spec_http_path:
-            self.context.http_server.register_handler(path, self.get_specification)
+        self._spec_path = None
+        if self.spec_path:
+            self.context.http_server.register_handler(self.spec_path, self.get_specification)
+            if self.ui_path:
+                self.context.http_server.register_handler(self.ui_path, self.get_ui)
+                ui_sub_path = yarl.URL(self.ui_path).join(yarl.URL('{file_name}')).path
+                self.context.http_server.register_handler(ui_sub_path, self.get_ui)
+        elif self.ui_path:
+            logger.warning('Swagger UI disabled due to disabled spec endpoint')
+
+    async def get_ui(self, request: IncomingRequest) -> FileAnswer:
+        file_name = request.key_value_params.get('file_name', 'index.html')
+        target = pathlib.Path(__file__).parent / 'swagger-ui' / file_name
+        if file_name.endswith('.html'):
+            content = target.read_text()
+            content_type = ContentType.HTML
+        elif file_name.endswith('.css'):
+            content = target.read_text()
+            content_type = ContentType.Css
+        elif file_name.endswith('.js'):
+            content = target.read_text()
+            content_type = ContentType.JavaScript
+        elif file_name.endswith('.png'):
+            content = target.read_bytes()
+            content_type = ContentType.PNG
+        else:
+            raise TypeError(f"Unsupported file type: {file_name}")
+        if file_name == 'swagger-initializer.js':
+            content = content.replace('https://petstore.swagger.io/v2/swagger.json', self.spec_path)
+        return FileAnswer(content, content_type=content_type)
 
     async def get_specification(self, _: IncomingRequest) -> FileAnswer:
         content = custom_dumps(self.wrapper.get_spec_dict(), indent=2)
